wkGeoset geoset;
wkView view;

#include "wcDialogConstant.h"

#define ERROR_LAYER_NOT_FOUND 			0x0001
#define ERROR_FIELD_NOT_FOUND			0x0002
#define ERROR_FIELD_VALUE_EMPTY			0x0003
#define ERROR_NO_ERROR				0x0004

bool Respond(wkDialog& dlg, int ctrlId, int action) {
	if (ctrlId == 1002 && action == wdMsgBnClicked)
		check(dlg);
	if (ctrlId == 1001 && action == wdMsgSelchange) {
		int n = dlg.getCurSel(1001);
		if (n >= 0) {
			string sLayer = dlg.getItemText(1001, n, 1);
			string sId = dlg.getItemText(1001, n, 2);
			if (!sId.isEmpty()) {
				int id = parseInt(sId);
				wkLayer layer = geoset.getLayer(sLayer);
				wkFeature fea = layer.getFeature(id);
				wkGeometry geo = fea.getGeometry();
				view.boxView(geo.bounds(), 1);
				view.refresh();
			}
		}
	}
	if (ctrlId == 1003 && action == wdMsgBnClicked)
		check(dlg);
}

void CreateDlg(wkDialog& dlg) {
	dlg.create("房产测量属性检查");
	dlg.addCtrl(1001, wdCtrlListCtrl, 5, 5, 160, 300,
			LVS_SINGLESEL | LVS_SHOWSELALWAYS | LVS_NOSORTHEADER,
			LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP | LVS_EX_GRIDLINES);
	dlg.insertColumn(1001, 0, "#", LVCFMT_LEFT, 50);
	dlg.insertColumn(1001, 1, "图层", LVCFMT_LEFT, 50);
	dlg.insertColumn(1001, 2, "ID", LVCFMT_LEFT, 40);
	dlg.insertColumn(1001, 3, "字段", LVCFMT_LEFT, 80);
	dlg.insertColumn(1001, 4, "检查结果", LVCFMT_LEFT, 150);
	dlg.setAction(1001, wdMsgSelchange);
	dlg.addCtrl(1002, wdCtrlButton, 5, 330, 160, 20, 0, "刷新");
	dlg.addCtrl(1003, wdCtrlCheck, 5, 305, 150, 20, 0, "仅显示错误");
	dlg.setCheck(1003, 1);
	dlg.setAction(1002, wdMsgBnClicked);
	dlg.setAction(1003, wdMsgBnClicked);
	check(dlg);
}

void main(array args) {
	wkDialog dlg;
	if (args.getSize() < 3) {
		CreateDlg(dlg);
	} else if (args[args.getSize() - 3] == "Respond") {
		dlg.from((args[args.getSize()-3])); //对话框
		if (dlg.handle()==0)
			return;

		//callFuncByName("Respond", dlg, ctrlId, action);
		callFuncByName("Respond", dlg,
			atoi(args[args.getSize()-2]), atoi(args[args.getSize()-1]));
	}
}

void check(wkDialog& dlg) {
	dlg.clear(1001);
	wkDb db = geoset.getDbAt(0);
	array oLayerFlds = { { "宗地", "房屋坐落", "东", "西", "南", "北", "建筑结构", "建成年份",
			"产权人" }, { "幢", "幢号", "建筑结构", "总层次", "建成年份" }, { "房屋", "lszd",
			"fwjg", "fwcs", "zldz" } };

	int count = 0;
	for (int i = 0; i < oLayerFlds.getSize(); ++i) {
		string sLayer = oLayerFlds[i][0];
		for (int j = 1; j < oLayerFlds[i].getSize(); ++j) {
			string sField = oLayerFlds[i][j];
			array errs;
			int srt = CheckFldsNotNull(db, sLayer, sField, errs);
			//amessage(sLayer+","+sField+","+toString(srt)+","+toString(errs));
			if (srt == ERROR_LAYER_NOT_FOUND) {
				dlg.insertItem(1001, count, toString(count));
				dlg.setItemText(1001, count, 1, errs[0]);
				dlg.setItemText(1001, count, 2, "");
				dlg.setItemText(1001, count, 3, "");
				dlg.setItemText(1001, count, 4, "层不存在");
				++count;
				break;
			}
			if (srt == ERROR_FIELD_NOT_FOUND) {
				dlg.insertItem(1001, count, toString(count));
				dlg.setItemText(1001, count, 1, errs[0]);
				dlg.setItemText(1001, count, 2, "");
				dlg.setItemText(1001, count, 3, errs[1]);
				dlg.setItemText(1001, count, 4, "字段不存在");
				++count;
			}
			if (srt == ERROR_NO_ERROR) {
				if (!dlg.getCheck(1003)) {
					dlg.insertItem(1001, count, toString(count));
					dlg.setItemText(1001, count, 1, errs[0]);
					dlg.setItemText(1001, count, 2, "");
					dlg.setItemText(1001, count, 3, errs[1]);
					dlg.setItemText(1001, count, 4, "无错误");
					++count;
				}
			}
			if (srt == ERROR_FIELD_VALUE_EMPTY) {
				for (int m = 0; m < errs[2].getSize(); ++m) {
					dlg.insertItem(1001, count, toString(count));
					dlg.setItemText(1001, count, 1, errs[0]);
					dlg.setItemText(1001, count, 2, errs[2][m]);
					dlg.setItemText(1001, count, 3, errs[1]);
					dlg.setItemText(1001, count, 4, "字段空值");
					++count;
				}
			}
		}
	}

	if (count == 0) {
		dlg.show(0, 0);
		amessage("属性检查完成，无错误");
	}
}

int CheckFldsNotNull(wkDb& db, string& sLayer, string& sField,
		array& errorDatas) {
	errorDatas.removeAll();
	if (!db.isLayerExisted(sLayer)) {
		errorDatas.add(sLayer);
		return ERROR_LAYER_NOT_FOUND;
	}

	string sErrNote, sErrAtt;
	string sTable = db.getLayerTableName(sLayer);
	if (db.getFieldSQLType(sTable, sField) == 0) {
		errorDatas.add(sLayer);
		errorDatas.add(sField);
		return ERROR_FIELD_NOT_FOUND;
	}

	string sWhere;
	sWhere.format("[%s] is null or [%s]=''", sField, sField);

	string sql;
	sql.format("select [FeatureID] from [%s] where %s", sTable, sWhere);
	array vals = db.getDataBySQL(sql, 1000000);
	if (vals.getSize() == 0) {
		errorDatas.add(sLayer);
		errorDatas.add(sField);
		return ERROR_NO_ERROR;
	}

	errorDatas.add(sLayer);
	errorDatas.add(sField);
	errorDatas.add(vals);
	return ERROR_FIELD_VALUE_EMPTY;
}
