namespace WalkScript.MSExcel
{
public:
	/// <summary>
	/// Open an existing workbook.
	/// </summary>
	/// <param name="sfileName">Path and filename of the file to be opened</param>
	/// <param name="bVisible">Flag, whether to show or hide the workbook (0=not visible, 1=visible) (default=0)</param>
	/// <returns>0ļڣ޷ļ1򿪳ɹ</returns>
	int Open(CString sfileName, bool bVisible = false);

	/// <summary>
	/// Closes the active workbook and removes the specified Excel object.
	/// </summary>
	/// <param name="bSave">Flag for saving the file before closing (0=no save, 1=save) (default = 1)</param>
	void Close(bool bSave = false);

	/// <summary>
	/// Saves the active workbook of the specified Excel object.
	/// </summary>
	void Save();

	/// <summary>
	/// Saves the active workbook of the specified Excel object with a new filename and type.
	/// </summary>
	/// <param name="filePath">Path and filename of the file to be read</param>
	/// <param name="fileType">Excel writable filetype string = "xls|xlsx|csv|txt|template|html|xml", default "xls"</param>
	void SaveAs(CString filePath, CString fileType);

	CString RowCol2Range(int startRow, int startCol, int nRows, int nCols);

	/// <summary>
	/// Write information to a cell on the active worksheet.
	/// </summary>
	/// <param name="sValue">Value to be written</param>
	/// <param name="row">The row to write</param>
	/// <param name="column">The column to write</param>
	/// <returns></returns>
	int SetCellString(CString sValue, int row, int column);

	/// <summary>
	/// Write information to a cell on the active worksheet.
	/// </summary>
	/// <param name="fValue">Value to be written</param>
	/// <param name="row">The row to write</param>
	/// <param name="column">The column to write</param>
	/// <returns></returns>
	int SetCellDouble(double fValue, int row, int column);

	/// <summary>
	/// Write information to a cell on the active worksheet.
	/// </summary>
	/// <param name="nValue">Value to be written</param>
	/// <param name="row">The row to write</param>
	/// <param name="column">The column to write</param>
	/// <returns></returns>
	int SetCellInt(int nValue, int row, int column);

	/// <summary>
	/// Write information to a cell on the active worksheet
	/// </summary>
	/// <param name="sValue">Value to be written</param>
	/// <param name="sRange">range to write </param>
	/// <returns></returns>
	int SetRange(CString sValue, CString sRange);

	int SetRange(int nValue, CString sRange);

	int SetRange(double fValue, CString sRange);

	/// <summary>
	/// Write a formula to a cell on the active worksheet 
	/// </summary>
	/// <param name="sFormula">Formula to be written</param>
	/// <param name="row">row of cell</param>
	/// <param name="column">column of cell</param>
	/// <returns></returns>
	int SetCellFormula(CString sFormula, int row, int column);

	/// <summary>
	/// Write a formula to a range on the active worksheet 
	/// </summary>
	/// <param name="sFormula">Formula to be written</param>
	/// <param name="sRange">range</param>
	/// <returns></returns>
	int SetRangeFormula(CString sFormula, CString sRange);

	/// <summary>
	/// Read information from the active worksheet 
	/// </summary>
	/// <param name="row"></param>
	/// <param name="column"></param>
	/// <returns></returns>
	CString GetCellString(int row, int column);

	/// <summary>
	/// Read information from the active worksheet 
	/// </summary>
	/// <param name="row"></param>
	/// <param name="column"></param>
	/// <returns></returns>
	double GetCellDouble(int row, int column);

	/// <summary>
	/// Read information from the active worksheet 
	/// </summary>
	/// <param name="row"></param>
	/// <param name="column"></param>
	/// <returns></returns>
	int GetCellInt(int row, int column);

	/// <summary>
	/// Delete a number of rows from the active worksheet.
	/// </summary>
	/// <param name="row">The specified row number to delete</param>
	/// <param name="numRows">he number of rows to delete</param>
	/// <returns></returns>
	int DeleteRow(int row, int numRows = 1);

	/// <summary>
	/// Delete a number of columns from the active worksheet.
	/// </summary>
	/// <param name="column">The specified column number to delete</param>
	/// <param name="numCols">The number of columns to delete</param>
	/// <returns></returns>
	int DeleteColumn(int column, int numCols = 1);

	/// <summary>
	/// Insert a number of rows into the active worksheet.
	/// </summary>
	/// <param name="row">The specified row number to inert</param>
	/// <param name="numRows">The number of rows to insert</param>
	/// <returns></returns>
	int InsertRow(int row, int numRows = 1);

	/// <summary>
	/// Insert a number of columns into the active worksheet.
	/// </summary>
	/// <param name="column">The specified column number to insert</param>
	/// <param name="numCols">The number of columns to insert</param>
	/// <returns></returns>
	int InsertColumn(int column, int numCols = 1);

	/// <summary>
	/// Add new sheet to workbook - optionally with a name.
	/// </summary>
	/// <param name="sName">The name of the sheet to create (default follows standard Excel new sheet convention)</param>
	/// <returns></returns>
	int AddSheet(CString sName = "");

	/// <summary>
	/// Delete the specified sheet by string name or by number.
	/// </summary>
	/// <param name="nWhich"></param>
	/// <param name="fAlerts"></param>
	/// <returns></returns>
	int DeleteSheet(int nWhich, bool bAlerts = false);

	/// <summary>
	/// Activate the specified sheet by number.
	/// </summary>
	/// <param name="iSheet">The sheet to activate </param>
	/// <returns>1Success0Failedsheet number does not exist.</returns>
	int ActiveSheet(int iSheet);

	/// <summary>
	/// ȡĿ
	/// </summary>
	/// <returns></returns>
	int GetWorkSheetCount();

	/// <summary>
	/// Return the name of the active sheet.
	/// </summary>
	/// <returns></returns>
	CString GetActiveSheetName();

	/// <summary>
	/// Set the name of the active sheet.
	/// </summary>
	/// <param name="sName"></param>
	void SetActiveSheetName(CString sName);

	/// <summary>
	/// excelвͼƬ
	/// </summary>
	/// <param name="sImgFile"></param>
	/// <returns>ڵ</returns>
	int AddImage(int row, int col, CString sImgFile, int nRows = -1, int nCols = -1, bool bStretch = false);
	int AddImage(CString sRange, CString sImgFile, bool bStretch = false);

	/// <summary>
	/// ƳеͼƬ
	/// </summary>
	/// <param name="nImgIndex">ѲͼƬֵǴAddImageʱصõ</param>
	void RemoveImage(int nImgIndex);

	/// <summary>
	/// The macro to run. This can be either a string with the macro
	//     name, a Microsoft.Office.Interop.Excel.Range object indicating where the
	//     function is, or a register ID for a registered DLL (XLL) function. If a string
	//     is used, the string will be evaluated in the context of the active sheet.
	/// </summary>
	/// <param name="sMacro"></param>
	void Run(CString sMacro);

	/// <summary>
	/// Return active sheet rows count.
	/// </summary>
	/// <returns></returns>
	int GetActiveSheetRows();

	/// <summary>
	/// Return active sheet columns count.
	/// </summary>
	/// <returns></returns>
	int GetActiveSheetColumns();

	/// <summary>
	/// Merge active sheet at cell by rows.
	/// </summary>
	/// <param name="iStartRow"></param>
	/// <param name="iStartCol"></param>
	/// <param name="rows"></param>
	void MergeRows(int iStartRow, int iStartCol, int rows);

	/// <summary>
	/// Merge active sheet at cell by columns.
	/// </summary>
	/// <param name="iStartRow"></param>
	/// <param name="iStartCol"></param>
	/// <param name="cols"></param>
	void MergeCols(int iStartRow, int iStartCol, int cols);

	/// <summary>
	/// Merge active sheet by range. eg, Merge("A1:B3");
	/// </summary>
	/// <param name="sRange"></param>
	void Merge(CString sRange);

	bool NewBook(bool bVisible = false);

	/// <summary>
	/// </summary>
	/// <param name="connection">
	//  The data source for the query table. Can be one of the following:A
	//     string containing an OLE DB or ODBC connection string. The ODBC connection
	//     string has the form "ODBC;<connection string>".A QueryTable object from which
	//     the query information is initially copied, including the connection string
	//     and the SQL text, but not including the Destination range. Specifying a QueryTable
	//     object causes the Sql argument to be ignored.An ADO or DAO Recordset object.
	//     Data is read from the ADO or DAO recordset. Microsoft Excel retains the recordset
	//     until the query table is deleted or the connection is changed. The resulting
	//     query table cannot be edited.A Web query. A string in the form "URL;<url>",
	//     where "URL;" is required but not localized and the rest of the string is
	//     used for the URL of the Web query.Data Finder. A string in the form "FINDER;<data
	//     finder file path>" where "FINDER;" is required but not localized. The rest
	//     of the string is the path and file name of a Data Finder file (*.dqy or *.iqy).
	//     The file is read when the Add method is run; subsequent calls to the Microsoft.Office.Interop.Excel._QueryTable.Connection
	//     property of the query table will return strings beginning with "ODBC;" or
	//     "URL;" as appropriate.A text file. A string in the form "TEXT;<text file
	//     path and name>", where TEXT is required but not localized.
	/// </param>
	/// <param name="range">
	//  The cell in the upper-left corner of the query table destination
	//     range (the range where the resulting query table will be placed). The destination
	//     range must be on the worksheet that contains the Microsoft.Office.Interop.Excel.QueryTables
	//     object.
	/// </param>
	/// <param name="sql">
	//   The SQL query string to be run on the ODBC data source.
	//     This argument is optional when you're using an ODBC data source You cannot
	//     use this argument when a QueryTable object, text file, or ADO or DAO Recordset
	//     object is specified as the data source.
	/// </param>
	/// <returns></returns>
	bool FillRangeBySQL(CString connection, CString range, CString sql, bool bShowFieldNames=false);
	bool FillCellBySQL(CWalkOdbcBase* pWob, CString sSQL, CString sRange);

	void Hide(CString range, bool bHide = true);

	/// <summary>
	/// Set excel font
	/// </summary>
	/// <param name="sRange">excel range</param>
	/// <param name="sFontName">Font name, like 塱Arial塱 and so on default=塱</param>
	/// <param name="nSize">font size default is 12</param>
	/// <param name="clrColor">font Color default = 0black</param>
	/// <param name="bBold">default is false</param>
	/// <param name="sFontStyle">default is 桱 can be Ӵ֡б桢Ӵб...</param>
	/// <param name="bItalic">default is false</param>
	/// <param name="bStrikethrough">default is false</param>
	/// <param name="bShadow">default is false</param>
	/// <param name="bOutlineFont">default is false</param>
	/// <param name="nUnderline">
	/// default is -4142xlUnderlineStyleNone     »
	///     can be 2xlUnderlineStyleSingle       »
	///            -4119xlUnderlineStyleDouble     ˫»
	/// </param>       
	/// <param name="bSubscript">default is false</param>
	/// <param name="bSuperscript">default is false</param>
	void SetFont(CString sRange, CString sFontName = "", int nSize = 12,
		int clrColor = 0, bool bBold = false, 
		bool bItalic = false, bool bStrikethrough = false, bool bShadow = false,
		bool bOutlineFont = false, int nUnderline = -4142 /*xlUnderlineStyleNone*/,
		bool bSubscript = false, bool bSuperscript = false);

	/// <summary>
	/// Set cell alignment
	/// </summary>
	/// <param name="sRange">Range of sheet</param>
	/// <param name="nHorizontalAlignment">
	/// xlGeneral = 1( default),<br/>
	/// xlLeft = -4131, <br/>
	/// xlCenter = -4108, <br/>
	/// xlRight = -4152
	/// </param>
	/// <param name="nVerticalAlignment">
	/// xlTop = -4160,
	/// xlCenter = -4108,        
	/// xlBottom = -4107,
	/// </param>
	void SetAlignment(CString sRange,
		int nHorizontalAlignment = 1 /*xlGeneral*/,
		int nVerticalAlignment = -4108 /*xlCenter*/);

	/// <summary>
	/// ɫ
	/// </summary>
	/// <param name="sRange">ԪΧ</param>
	/// <param name="color">ɫ</param>
	void SetFontColor(CString sRange, int color);

	/// <summary>
	/// õԪ񱳾ɫ
	/// </summary>
	/// <param name="sRange">ԪΧ</param>
	/// <param name="color">ɫ</param>
	void SetCellBgColor(CString sRange, int color);

	/// <summary>
	/// ɾԪ
	/// </summary>
	/// <param name="iStartRow">ʼ</param>
	/// <param name="iStartCol">ʼ</param>
	/// <param name="shiftType">ƶ, ShiftUp = 1, ShiftToLeft = 2</param>
	/// <param name="nRows">ɾռ</param>
	/// <param name="nCols">ɾռ</param>
	void DeleteCells(int iStartRow, int iStartCol, int shiftType = 1, int nRows = 1, int nCols = 1);

	/// <summary>
	/// ӡļ
	/// </summary>
	/// <param name="sPrintToFileName">ļ</param>
	/// <param name="from">ʼҳ</param>
	/// <param name="to">ҳ</param>
	/// <param name="nCopys"></param>
	void Print(CString sPrintToFileName = "", int from=1, int to=-1, int nCopys=1);

	/// <summary>
	/// ôӡ
	/// </summary>
	/// <param name="sPrinterName">ӡ</param>
	/// <returns>ԭȵĴӡ</returns>
	CString ActivePrinter(CString sPrinterName);

	/// <summary>
	/// ҳü
	/// </summary>
	/// <param name="nPos">ҳüλá1-Left2-Center3-Right</param>
	/// <param name="sval">ҳü </param>
	void SetHeader(int nPos, CString sVal);

	/// <summary>
	/// ȡҳü
	/// </summary>
	/// <param name="nPos">ҳüλá1-Left2-Center3-Right</param>
	/// <returns>ҳü </returns>
	CString GetHeader(int nPos);

	/// <summary>
	/// ҳ
	/// </summary>
	/// <param name="nPos">ҳλá1-Left2-Center3-Right</param>
	/// <param name="sval">ҳ </param>
	void SetFooter(int nPos, CString sVal);

	/// <summary>
	/// ȡҳ
	/// </summary>
	/// <param name="nPos">ҳλá1-Left2-Center3-Right</param>
	/// <returns>ҳ </returns>
	CString GetFooter(int nPos);

}